@@ -9,7 +9,6 @@ use std::{
 
 use clap::{crate_authors, crate_description, value_parser, Arg, ArgAction, Command};
 use crossterm::tty::IsTty;
-use itertools::Itertools;
 
 use crate::{
     display::style::BackgroundColor,
@@ -229,7 +228,7 @@ json: Output the results as a machine-readable JSON array with an element per fi
                 .value_parser(["on", "off"])
                 .default_value("on")
                 .action(ArgAction::Set)
-                .help("Remove any carriage return characters before diffing. This can be helpful when dealing with files on Windows that contain CRLF, i.e. `\\r\\n`.\n\nWhen disabled, difftastic will consider multiline string literals (in code) or mutiline text (e.g. in HTML) to differ if the two input files have different line endings.")
+                .help("Remove any carriage return characters before diffing. This can be helpful when dealing with files on Windows that contain CRLF, i.e. `\\r\\n`.\n\nWhen disabled, difftastic will consider multiline string literals (in code) or multiline text (e.g. in HTML) to differ if the two input files have different line endings.")
         )
         .arg(
             Arg::new("check-only").long("check-only")
@@ -247,7 +246,7 @@ json: Output the results as a machine-readable JSON array with an element per fi
             Arg::new("skip-unchanged").long("skip-unchanged")
                 .action(ArgAction::SetTrue)
                 .env("DFT_SKIP_UNCHANGED")
-                .help("Don't display anything if a file is unchanged.")
+                .help("Don't display anything if a file is unchanged. This is useful when comparing directories of files.")
         )
         .arg(
             Arg::new("override").long("override")
@@ -275,13 +274,13 @@ When multiple overrides are specified, the first matching override wins."))
         .arg(
             Arg::new("list-languages").long("list-languages")
                 .action(ArgAction::SetTrue)
-                .help("Print all the languages supported by difftastic, along with their extensions.")
+                .help("Print all the languages supported by difftastic, along with their recognised extensions.")
         )
         .arg(
             Arg::new("byte-limit").long("byte-limit")
                 .value_name("LIMIT")
                 .action(ArgAction::Set)
-                .help("Use a text diff if either input file exceeds this size.")
+                .help("Use a line-oriented diff if either input file exceeds this size.")
                 .default_value(format!("{}", DEFAULT_BYTE_LIMIT))
                 .env("DFT_BYTE_LIMIT")
                 .value_parser(clap::value_parser!(usize))
@@ -290,7 +289,9 @@ When multiple overrides are specified, the first matching override wins."))
         .arg(
             Arg::new("graph-limit").long("graph-limit")
                 .value_name("LIMIT")
-                .help("Use a text diff if the structural graph exceed this number of nodes in memory.")
+                .help("Use a line-oriented diff if the internal graph exceeds this number of vertices. This limit controls the worst case runtime and memory usage for difftastic.
+
+Higher values will allow difftastic to perform a structural diff in more cases. Higher values will also increase the time before difftastic gives up on structural diffing, and increase peak memory usage.")
                 .default_value(format!("{}", DEFAULT_GRAPH_LIMIT))
                 .action(ArgAction::Set)
                 .env("DFT_GRAPH_LIMIT")
@@ -301,7 +302,7 @@ When multiple overrides are specified, the first matching override wins."))
             Arg::new("parse-error-limit").long("parse-error-limit")
                 .value_name("LIMIT")
                 .action(ArgAction::Set)
-                .help("Use a text diff if the number of parse errors exceeds this value.")
+                .help("Use a line-oriented diff if the number of parse errors exceeds this value.")
                 .default_value(format!("{}", DEFAULT_PARSE_ERROR_LIMIT))
                 .env("DFT_PARSE_ERROR_LIMIT")
                 .value_parser(clap::value_parser!(usize))
@@ -612,19 +613,20 @@ fn parse_overrides_or_die(raw_overrides: &[String]) -> Vec<(LanguageOverride, Ve
         std::process::exit(EXIT_BAD_ARGUMENTS);
     }
 
-    overrides
-        .into_iter()
-        .coalesce(
-            |(prev_lang, mut prev_globs), (current_lang, current_globs)| {
-                if prev_lang == current_lang {
-                    prev_globs.extend(current_globs);
-                    Ok((prev_lang, prev_globs))
-                } else {
-                    Err(((prev_lang, prev_globs), (current_lang, current_globs)))
-                }
-            },
-        )
-        .collect()
+    let mut combined_overrides: Vec<(LanguageOverride, Vec<glob::Pattern>)> = vec![];
+    for (lang, globs) in overrides {
+        if let Some((prev_lang, prev_globs)) = combined_overrides.last_mut() {
+            if *prev_lang == lang {
+                prev_globs.extend(globs);
+            } else {
+                combined_overrides.push((lang, globs));
+            }
+        } else {
+            combined_overrides.push((lang, globs));
+        }
+    }
+
+    combined_overrides
 }
 
 /// Parse CLI arguments passed to the binary.
